unit eff_button;
//EFFECT Button
//Created by Yakovchenko Sergey 2008
interface
uses
Forms,Windows,Messages,Graphics,
ExtCtrls,Classes,SysUtils,Variants,Dialogs,Controls,StdCtrls;
type
TEffectButton = class(TCustomControl)
private
FLeaveBackground: TPIcture;
FEnterBackground: TPicture;
FInterval: Integer;
FText: String;
FFont: TFont;
MouseEnter: Boolean;
procedure SetLeaveBackground(Value: TPicture);
procedure SetEnterBackground(Value: TPicture);
procedure SetFont(Value: TFont);
procedure SetInterval(Value: Integer);
procedure SetText(Value: String);
procedure WMLeave(var Message: TMessage); message CM_MOUSELEAVE;
procedure WMEnter(var Message: TMessage); message CM_MOUSEENTER;
procedure Morphing(Bm1,Bm2: TBitmap);
public
procedure Paint; override;
constructor Create(AOWner: TComponent); override;
destructor Destroy; override;

published
property onClick;
property Font;
property Caption: String read FText write SetText;
property LeaveImage: TPicture read FLeaveBackground write SetLeaveBackground;
property EnterImage: TPicture read FEnterBackground write SetEnterBackground;
property Interval: Integer read FInterval write SetInterval;
property ShowHint;
end;


procedure Register;

implementation
procedure Register;
begin
RegisterComponents('DelphiExpert.ru',[TEffectButton]);

end;
procedure TEffectButton.SetFont(Value: TFont);
begin
FFont.Assign(Value);
end;
procedure TEffectButton.SetLeaveBackground(Value: TPicture);
begin
FLeaveBackground.Assign(Value);
Invalidate;
end;
procedure TEffectButton.SetEnterBackground(Value: TPicture);
begin
FEnterBackground.Assign(Value);
Invalidate;
end;
procedure TEffectButton.SetInterval(Value: Integer);
begin
if Value < 0 then Value:= 0;
FInterval:= Value;
end;
procedure TEffectButton.SetText(Value: String);
begin
FText:= Value;
end;
constructor TEffectButton.Create(AOWner: TComponent);
begin
inherited;
FLeaveBackground:= TPicture.Create;
FEnterBackground:= TPicture.Create;
Parent:= TForm(AOWner);
Parent.DoubleBuffered:= True;
FInterval:= 300;

end;
destructor TEffectButton.Destroy;
begin
inherited;
FLeaveBackground.Destroy;
FEnterBackground.Destroy;

end;
procedure TEffectButton.Morphing(Bm1,Bm2: TBitmap);

var
  i: integer;
  x, y: integer;
  bm: TBitMap;
  p1, p2, p: PByteArray;
  c: integer;
  k: integer;
begin
  bm := TBitMap.Create;

if bm1.Height < bm2.Height then
  begin
    bm.Height := bm1.Height;
    bm2.Height := bm1.Height;
  end
  else
  begin
    bm.Height := bm2.Height;
    bm1.Height := bm2.Height;
  end;
  if bm1.Width < bm2.Width then
  begin
    bm.Width := bm1.Width;
    bm2.Width := bm1.Width;
  end
  else
  begin
    bm.Width := bm2.Width;
    bm1.Width := bm2.Width;
  end;

  bm.PixelFormat := pf24bit;
  bm1.PixelFormat := pf24bit;
  bm2.PixelFormat := pf24bit;

    Canvas.Draw(0,0,Bm1);
    SetBkMode(Canvas.Handle,TRANSPARENT);
Canvas.TextOut((Width div 2) - (Canvas.TextWidth(FText) div 2),(Height div 2) - (Canvas.TextHeight(FText) div 2),FText);
SetBkMode(Canvas.Handle,OPAQUE);
  for i := 1 to FInterval - 1 do
  begin
    for y := 0 to bm.Height - 1 do
    begin
      p := bm.ScanLine[y];
      p1 := bm1.ScanLine[y];
      p2 := bm2.ScanLine[y];
      for x := 0 to bm.Width * 3 - 1 do
        p^[x] := round((p1^[x] * (FInterval - i) + p2^[x] * i) / FInterval);
    end;
        Canvas.Draw(0,0,Bm);
        SetBkMode(Canvas.Handle,TRANSPARENT);
Canvas.TextOut((Width div 2) - (Canvas.TextWidth(FText) div 2),(Height div 2) - (Canvas.TextHeight(FText) div 2),FText);
SetBkMode(Canvas.Handle,OPAQUE);
    Application.ProcessMessages;
    if Application.Terminated then
      break;
  end;
   Canvas.Draw(0,0,Bm2);
   SetBkMode(Canvas.Handle,TRANSPARENT);
Canvas.TextOut((Width div 2) - (Canvas.TextWidth(FText) div 2),(Height div 2) - (Canvas.TextHeight(FText) div 2),FText);
SetBkMode(Canvas.Handle,OPAQUE);
  bm.Destroy;
end;



procedure TEffectButton.WMLeave(var Message: TMessage);


 begin

  inherited;



   if FEnterBackground.Bitmap.Handle = 0 then Exit;

   Morphing(FEnterBackground.Bitmap,FLeaveBackground.Bitmap);





end;
procedure TEffectButton.WMEnter(var Message: TMessage);


 begin





     if FEnterBackground.Bitmap.Handle = 0 then Exit;

   Morphing(FLeaveBackground.Bitmap,FEnterBackground.Bitmap);



end;
procedure TEffectButton.Paint;
begin
inherited;
Canvas.Font:= Font;
SetBkMode(Canvas.Handle,TRANSPARENT);
Canvas.TextOut((Width div 2) - (Canvas.TextWidth(FText) div 2),(Height div 2) - (Canvas.TextHeight(FText) div 2),FText);
SetBkMode(Canvas.Handle,OPAQUE);
if FLeaveBackground.Bitmap.Handle = 0 then Exit;
Canvas.Draw(0,0,FLeaveBackground.Bitmap);
SetBkMode(Canvas.Handle,TRANSPARENT);
Canvas.TextOut((Width div 2) - (Canvas.TextWidth(FText) div 2),(Height div 2) - (Canvas.TextHeight(FText) div 2),FText);
SetBkMode(Canvas.Handle,OPAQUE);

end;

end.
